@@ -89,16 +89,16 @@
  * <refsect2>
  * <title>Examples</title>
  * |[
- * gst-launch -v udpsrc ! fakesink dump=1
+ * gst-launch-1.0 -v udpsrc ! fakesink dump=1
  * ]| A pipeline to read from the default port and dump the udp packets.
  * To actually generate udp packets on the default port one can use the
  * udpsink element. When running the following pipeline in another terminal, the
  * above mentioned pipeline should dump data packets to the console.
  * |[
- * gst-launch -v audiotestsrc ! udpsink
+ * gst-launch-1.0 -v audiotestsrc ! udpsink
  * ]|
  * |[
- * gst-launch -v udpsrc port=0 ! fakesink
+ * gst-launch-1.0 -v udpsrc port=0 ! fakesink
  * ]| read udp packets from a free port.
  * </refsect2>
  *
@@ -362,14 +362,14 @@ gst_udpsrc_getcaps (GstBaseSrc * src, GstCaps * filter)
 static GstFlowReturn
 gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
 {
+  GstFlowReturn ret;
   GstUDPSrc *udpsrc;
   GstBuffer *outbuf;
+  GstMapInfo info;
   GSocketAddress *saddr = NULL;
-  guint8 *pktdata;
-  gint pktsize;
   gsize offset;
   gssize readsize;
-  gssize ret;
+  gssize res;
   gboolean try_again;
   GError *err = NULL;
 
@@ -422,13 +422,10 @@ retry:
   if (G_UNLIKELY (!readsize)) {
     /* try to read a packet (and it will be ignored),
      * in case a packet with no data arrived */
-
-    pktdata = NULL;
-    pktsize = 0;
-    ret =
-        g_socket_receive_from (udpsrc->used_socket, NULL, (gchar *) pktdata,
-        pktsize, udpsrc->cancellable, &err);
-    if (G_UNLIKELY (ret < 0))
+    res =
+        g_socket_receive_from (udpsrc->used_socket, NULL, NULL,
+        0, udpsrc->cancellable, &err);
+    if (G_UNLIKELY (res < 0))
       goto receive_error;
 
     /* poll again */
@@ -438,34 +435,47 @@ retry:
 no_select:
   GST_LOG_OBJECT (udpsrc, "ioctl says %d bytes available", (int) readsize);
 
-  pktdata = g_malloc (readsize);
-  pktsize = readsize;
+  ret = GST_BASE_SRC_CLASS (parent_class)->alloc (GST_BASE_SRC_CAST (udpsrc),
+      -1, readsize, &outbuf);
+  if (ret != GST_FLOW_OK)
+    goto alloc_failed;
+
+  gst_buffer_map (outbuf, &info, GST_MAP_WRITE);
   offset = 0;
 
   if (saddr)
     g_object_unref (saddr);
   saddr = NULL;
 
-  ret =
-      g_socket_receive_from (udpsrc->used_socket, &saddr, (gchar *) pktdata,
-      pktsize, udpsrc->cancellable, &err);
+  res =
+      g_socket_receive_from (udpsrc->used_socket, &saddr, (gchar *) info.data,
+      info.size, udpsrc->cancellable, &err);
 
-  if (G_UNLIKELY (ret < 0))
+  if (G_UNLIKELY (res < 0)) {
+    /* EHOSTUNREACH for a UDP socket means that a packet sent with udpsink
+     * generated a "port unreachable" ICMP response. We ignore that and try
+     * again. */
+    if (g_error_matches (err, G_IO_ERROR, G_IO_ERROR_HOST_UNREACHABLE)) {
+      gst_buffer_unmap (outbuf, &info);
+      gst_buffer_unref (outbuf);
+      outbuf = NULL;
+      g_clear_error (&err);
+      goto retry;
+    }
     goto receive_error;
+  }
 
-  /* patch pktdata and len when stripping off the headers */
+  /* patch offset and size when stripping off the headers */
   if (G_UNLIKELY (udpsrc->skip_first_bytes != 0)) {
     if (G_UNLIKELY (readsize < udpsrc->skip_first_bytes))
       goto skip_error;
 
     offset += udpsrc->skip_first_bytes;
-    ret -= udpsrc->skip_first_bytes;
+    res -= udpsrc->skip_first_bytes;
   }
 
-  outbuf = gst_buffer_new ();
-  gst_buffer_append_memory (outbuf,
-      gst_memory_new_wrapped (0, pktdata, pktsize, offset, ret, pktdata,
-          g_free));
+  gst_buffer_unmap (outbuf, &info);
+  gst_buffer_resize (outbuf, offset, res);
 
   /* use buffer metadata so receivers can also track the address */
   if (saddr) {
@@ -478,7 +488,7 @@ no_select:
 
   *buf = GST_BUFFER_CAST (outbuf);
 
-  return GST_FLOW_OK;
+  return ret;
 
   /* ERRORS */
 select_error:
@@ -500,9 +510,15 @@ get_available_error:
         ("get available bytes failed"));
     return GST_FLOW_ERROR;
   }
+alloc_failed:
+  {
+    GST_DEBUG ("Allocation failed");
+    return ret;
+  }
 receive_error:
   {
-    g_free (pktdata);
+    gst_buffer_unmap (outbuf, &info);
+    gst_buffer_unref (outbuf);
 
     if (g_error_matches (err, G_IO_ERROR, G_IO_ERROR_BUSY) ||
         g_error_matches (err, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
@@ -510,13 +526,16 @@ receive_error:
       return GST_FLOW_FLUSHING;
     } else {
       GST_ELEMENT_ERROR (udpsrc, RESOURCE, READ, (NULL),
-          ("receive error %" G_GSSIZE_FORMAT ": %s", ret, err->message));
+          ("receive error %" G_GSSIZE_FORMAT ": %s", res, err->message));
       g_clear_error (&err);
       return GST_FLOW_ERROR;
     }
   }
 skip_error:
   {
+    gst_buffer_unmap (outbuf, &info);
+    gst_buffer_unref (outbuf);
+
     GST_ELEMENT_ERROR (udpsrc, STREAM, DECODE, (NULL),
         ("UDP buffer to small to skip header"));
     return GST_FLOW_ERROR;
@@ -724,6 +743,7 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
     if (!addr) {
       GList *results;
 
+      GST_DEBUG_OBJECT (src, "resolving IP address for host %s", src->host);
       resolver = g_resolver_get_default ();
       results =
           g_resolver_lookup_by_name (resolver, src->host, src->cancellable,
@@ -883,27 +903,27 @@ no_socket:
   }
 bind_error:
   {
-    gst_udpsrc_stop (GST_BASE_SRC (src));
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("bind failed: %s", err->message));
     g_clear_error (&err);
     g_object_unref (bind_saddr);
+    gst_udpsrc_stop (GST_BASE_SRC (src));
     return FALSE;
   }
 membership:
   {
-    gst_udpsrc_stop (GST_BASE_SRC (src));
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("could add membership: %s", err->message));
     g_clear_error (&err);
+    gst_udpsrc_stop (GST_BASE_SRC (src));
     return FALSE;
   }
 getsockname_error:
   {
-    gst_udpsrc_stop (GST_BASE_SRC (src));
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("getsockname failed: %s", err->message));
     g_clear_error (&err);
+    gst_udpsrc_stop (GST_BASE_SRC (src));
     return FALSE;
   }
 }
@@ -954,7 +974,7 @@ gst_udpsrc_stop (GstBaseSrc * bsrc)
 
       if (!g_socket_leave_multicast_group (src->used_socket,
               g_inet_socket_address_get_address (src->addr), FALSE,
-              src->multi_iface, NULL)) {
+              src->multi_iface, &err)) {
         GST_ERROR_OBJECT (src, "Failed to leave multicast group: %s",
             err->message);
         g_clear_error (&err);
