@@ -114,6 +114,50 @@ run_ns_app_loop (void) {
   [pool release];
 }
 
+static void
+gst_osx_videosink_check_main_run_loop (GstOSXVideoSink *sink)
+{
+  /* check if the main run loop is running */
+  gboolean is_running;
+
+  if (sink->mrl_check_done) {
+    return;
+  }
+  /* the easy way */
+  is_running = [[NSRunLoop mainRunLoop] currentMode] != nil;
+  if (is_running) {
+    goto exit;
+  } else {
+    /* the previous check doesn't always work with main loops that run
+     * cocoa's main run loop manually, like the gdk one, giving false
+     * negatives. This check defers a call to the main thread and waits to
+     * be awaken by this function. */
+    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
+    GstOSXVideoSinkObject * object = (GstOSXVideoSinkObject *) sink->osxvideosinkobject;
+    gint64 abstime;
+
+    g_mutex_lock (&sink->mrl_check_lock);
+    [object performSelectorOnMainThread:
+          @selector(checkMainRunLoop)
+          withObject:nil waitUntilDone:NO];
+    /* Wait 100 ms */
+    abstime = g_get_monotonic_time () + 100 * 1000;
+    is_running = g_cond_wait_until (&sink->mrl_check_cond,
+        &sink->mrl_check_lock, abstime);
+    g_mutex_unlock (&sink->mrl_check_lock);
+
+    [pool release];
+  }
+
+exit:
+  {
+  GST_DEBUG_OBJECT(sink, "The main runloop %s is running",
+      is_running ? "" : " not ");
+  sink->main_run_loop_running = is_running;
+  sink->mrl_check_done = TRUE;
+  }
+}
+
 static void
 gst_osx_video_sink_run_cocoa_loop (GstOSXVideoSink * sink )
 {
@@ -124,7 +168,7 @@ gst_osx_video_sink_run_cocoa_loop (GstOSXVideoSink * sink )
    * external NSView is passed to the sink through the GstVideoOverlay API,
    * we need to run the cocoa mainloop somehow.
    */
-  if ([[NSRunLoop mainRunLoop] currentMode] == nil) {
+  if (!sink->main_run_loop_running) {
 #ifdef RUN_NS_APP_THREAD
     /* run the main runloop in a separate thread */
 
@@ -143,10 +187,10 @@ gst_osx_video_sink_run_cocoa_loop (GstOSXVideoSink * sink )
         selector:@selector(nsAppThread) object:nil];
     [sink->ns_app_thread start];
 
-    g_mutex_lock (sink->loop_thread_lock);
+    g_mutex_lock (&sink->loop_thread_lock);
     while (!sink->app_started)
-      g_cond_wait (sink->loop_thread_cond, sink->loop_thread_lock);
-    g_mutex_unlock (sink->loop_thread_lock);
+      g_cond_wait (&sink->loop_thread_cond, &sink->loop_thread_lock);
+    g_mutex_unlock (&sink->loop_thread_lock);
 #else
   /* assume that there is a GMainLoop and iterate the main runloop from there
    */
@@ -205,6 +249,7 @@ gst_osx_video_sink_osxwindow_create (GstOSXVideoSink * osxvideosink, gint width,
   GST_INFO_OBJECT (osxvideosink, "'have-ns-view' message sent");
 
   osxvideosink->ns_app_thread = [NSThread mainThread];
+  gst_osx_videosink_check_main_run_loop (osxvideosink);
   gst_osx_video_sink_run_cocoa_loop (osxvideosink);
   [osxwindow->gstview setMainThread:osxvideosink->ns_app_thread];
 
@@ -459,9 +504,13 @@ gst_osx_video_sink_init (GstOSXVideoSink * sink)
   sink->superview = NULL;
   sink->osxvideosinkobject = [[GstOSXVideoSinkObject alloc] initWithSink:sink];
 #ifdef RUN_NS_APP_THREAD
-  sink->loop_thread_lock = g_mutex_new ();
-  sink->loop_thread_cond = g_cond_new ();
+  g_mutex_init (&sink->loop_thread_lock);
+  g_cond_init (&sink->loop_thread_cond);
 #endif
+  g_mutex_init (&sink->mrl_check_lock);
+  g_cond_init (&sink->mrl_check_cond);
+  sink->mrl_check_done = FALSE;
+  sink->main_run_loop_running = FALSE;
   sink->app_started = FALSE;
   sink->keep_par = FALSE;
 }
@@ -490,6 +539,9 @@ gst_osx_video_sink_finalize (GObject *object)
   if (osxvideosink->osxvideosinkobject)
     [(GstOSXVideoSinkObject*)(osxvideosink->osxvideosinkobject) release];
 
+  g_mutex_clear (&osxvideosink->mrl_check_lock);
+  g_cond_clear (&osxvideosink->mrl_check_cond);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -537,7 +589,7 @@ gst_osx_video_sink_class_init (GstOSXVideoSinkClass * klass)
   g_object_class_install_property (gobject_class, ARG_FORCE_PAR,
       g_param_spec_boolean ("force-aspect-ratio", "force aspect ration",
           "When enabled, scaling will respect original aspect ration",
-          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          TRUE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -785,7 +837,8 @@ gst_osx_video_sink_get_type (void)
   NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 
   GST_INFO_OBJECT (osxvideosink, "resizing");
-  [osxwindow->gstview setVideoSize:osxwindow->width :osxwindow->height];
+  NSSize size = {osxwindow->width, osxwindow->height};
+  [osxwindow->win setContentSize:size];
   GST_INFO_OBJECT (osxvideosink, "done");
 
   [pool release];
@@ -861,10 +914,10 @@ gst_osx_video_sink_get_type (void)
   [NSApplication sharedApplication];
   [NSApp finishLaunching];
 
-  g_mutex_lock (sink->loop_thread_lock);
+  g_mutex_lock (&sink->loop_thread_lock);
   sink->app_started = TRUE;
-  g_cond_signal (sink->loop_thread_cond);
-  g_mutex_unlock (sink->loop_thread_lock);
+  g_cond_signal (&sink->loop_thread_cond);
+  g_mutex_unlock (&sink->loop_thread_lock);
 
   /* run the loop */
   run_ns_app_loop ();
@@ -873,6 +926,13 @@ gst_osx_video_sink_get_type (void)
 }
 #endif
 
+-(void) checkMainRunLoop
+{
+  g_mutex_lock (&osxvideosink->mrl_check_lock);
+  g_cond_signal (&osxvideosink->mrl_check_cond);
+  g_mutex_unlock (&osxvideosink->mrl_check_lock);
+}
+
 @end
 
 @ implementation GstBufferObject
