@@ -43,7 +43,6 @@ bus_handler (GstBus * bus, GstMessage * message, gpointer data)
       g_error ("error from %s: %s (%s)\n",
           GST_ELEMENT_NAME (GST_MESSAGE_SRC (message)), gerror->message,
           GST_STR_NULL (debug));
-      gst_message_unref (message);
       g_error_free (gerror);
       g_free (debug);
       g_main_loop_quit (loop);
@@ -125,11 +124,17 @@ GST_START_TEST (test_imagefreeze_0_1)
   GMainLoop *loop;
   guint n_buffers = G_MAXUINT;
   guint bus_watch = 0;
+  GstVideoInfo i1, i2;
 
-  caps1 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 25, 1, 1, 1);
-  caps2 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 0, 1, 1, 1);
+  gst_video_info_init (&i1);
+  gst_video_info_set_format (&i1, GST_VIDEO_FORMAT_xRGB, 640, 480);
+  i1.fps_n = 25;
+  i1.fps_d = 1;
+  caps1 = gst_video_info_to_caps (&i1);
+
+  gst_video_info_init (&i2);
+  gst_video_info_set_format (&i2, GST_VIDEO_FORMAT_xRGB, 640, 480);
+  caps2 = gst_video_info_to_caps (&i2);
 
   pipeline =
       setup_imagefreeze (caps1, caps2, G_CALLBACK (sink_handoff_cb_0_1),
@@ -188,11 +193,19 @@ GST_START_TEST (test_imagefreeze_25_1_0ms_400ms)
   GMainLoop *loop;
   guint n_buffers = G_MAXUINT;
   guint bus_watch = 0;
+  GstVideoInfo i1, i2;
 
-  caps1 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 25, 1, 1, 1);
-  caps2 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 25, 1, 1, 1);
+  gst_video_info_init (&i1);
+  gst_video_info_set_format (&i1, GST_VIDEO_FORMAT_xRGB, 640, 480);
+  i1.fps_n = 25;
+  i1.fps_d = 1;
+  caps1 = gst_video_info_to_caps (&i1);
+
+  gst_video_info_init (&i2);
+  gst_video_info_set_format (&i2, GST_VIDEO_FORMAT_xRGB, 640, 480);
+  i2.fps_n = 25;
+  i2.fps_d = 1;
+  caps2 = gst_video_info_to_caps (&i2);
 
   pipeline =
       setup_imagefreeze (caps1, caps2,
@@ -260,11 +273,19 @@ GST_START_TEST (test_imagefreeze_25_1_200ms_400ms)
   GMainLoop *loop;
   guint n_buffers = G_MAXUINT;
   guint bus_watch = 0;
+  GstVideoInfo i1, i2;
 
-  caps1 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 25, 1, 1, 1);
-  caps2 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 25, 1, 1, 1);
+  gst_video_info_init (&i1);
+  gst_video_info_set_format (&i1, GST_VIDEO_FORMAT_xRGB, 640, 480);
+  i1.fps_n = 25;
+  i1.fps_d = 1;
+  caps1 = gst_video_info_to_caps (&i1);
+
+  gst_video_info_init (&i2);
+  gst_video_info_set_format (&i2, GST_VIDEO_FORMAT_xRGB, 640, 480);
+  i2.fps_n = 25;
+  i2.fps_d = 1;
+  caps2 = gst_video_info_to_caps (&i2);
 
   pipeline =
       setup_imagefreeze (caps1, caps2,
@@ -332,11 +353,19 @@ GST_START_TEST (test_imagefreeze_25_1_400ms_0ms)
   GMainLoop *loop;
   guint n_buffers = G_MAXUINT;
   guint bus_watch = 0;
+  GstVideoInfo i1, i2;
 
-  caps1 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 25, 1, 1, 1);
-  caps2 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 25, 1, 1, 1);
+  gst_video_info_init (&i1);
+  gst_video_info_set_format (&i1, GST_VIDEO_FORMAT_xRGB, 640, 480);
+  i1.fps_n = 25;
+  i1.fps_d = 1;
+  caps1 = gst_video_info_to_caps (&i1);
+
+  gst_video_info_init (&i2);
+  gst_video_info_set_format (&i2, GST_VIDEO_FORMAT_xRGB, 640, 480);
+  i2.fps_n = 25;
+  i2.fps_d = 1;
+  caps2 = gst_video_info_to_caps (&i2);
 
   pipeline =
       setup_imagefreeze (caps1, caps2,
@@ -415,11 +444,19 @@ GST_START_TEST (test_imagefreeze_25_1_220ms_380ms)
   GMainLoop *loop;
   guint n_buffers = G_MAXUINT;
   guint bus_watch = 0;
+  GstVideoInfo i1, i2;
 
-  caps1 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 25, 1, 1, 1);
-  caps2 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 25, 1, 1, 1);
+  gst_video_info_init (&i1);
+  gst_video_info_set_format (&i1, GST_VIDEO_FORMAT_xRGB, 640, 480);
+  i1.fps_n = 25;
+  i1.fps_d = 1;
+  caps1 = gst_video_info_to_caps (&i1);
+
+  gst_video_info_init (&i2);
+  gst_video_info_set_format (&i2, GST_VIDEO_FORMAT_xRGB, 640, 480);
+  i2.fps_n = 25;
+  i2.fps_d = 1;
+  caps2 = gst_video_info_to_caps (&i2);
 
   pipeline =
       setup_imagefreeze (caps1, caps2,
@@ -460,104 +497,6 @@ GST_START_TEST (test_imagefreeze_25_1_220ms_380ms)
 
 GST_END_TEST;
 
-static GstBuffer *test_buffer = NULL;
-
-static GstFlowReturn
-test_bufferalloc (GstPad * pad, guint64 offset, guint size, GstCaps * caps,
-    GstBuffer ** buf)
-{
-  fail_if (test_buffer != NULL);
-
-  test_buffer = gst_buffer_new_and_alloc (size);
-  gst_buffer_set_caps (test_buffer, caps);
-
-  *buf = gst_buffer_ref (test_buffer);
-
-  return GST_FLOW_OK;
-}
-
-static void
-sink_handoff_cb_bufferalloc (GstElement * object, GstBuffer * buffer,
-    GstPad * pad, gpointer user_data)
-{
-  guint *n_buffers = (guint *) user_data;
-
-  if (*n_buffers == G_MAXUINT)
-    return;
-
-  fail_unless (buffer->parent != NULL);
-  fail_unless (test_buffer != NULL);
-  fail_unless (buffer->parent == test_buffer);
-
-  *n_buffers = *n_buffers + 1;
-}
-
-GST_START_TEST (test_imagefreeze_bufferalloc)
-{
-  GstElement *pipeline;
-  GstElement *sink;
-  GstPad *sinkpad;
-  GstCaps *caps1, *caps2;
-  GstBus *bus;
-  GMainLoop *loop;
-  guint n_buffers = G_MAXUINT;
-  guint bus_watch = 0;
-
-  caps1 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 25, 1, 1, 1);
-  caps2 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 25, 1, 1, 1);
-
-  pipeline =
-      setup_imagefreeze (caps1, caps2, G_CALLBACK (sink_handoff_cb_bufferalloc),
-      &n_buffers);
-
-  sink = gst_bin_get_by_name (GST_BIN (pipeline), "sink");
-  fail_unless (sink != NULL);
-  sinkpad = gst_element_get_static_pad (sink, "sink");
-  fail_unless (sinkpad != NULL);
-  gst_pad_set_bufferalloc_function (sinkpad, test_bufferalloc);
-  gst_object_unref (sinkpad);
-  gst_object_unref (sink);
-
-  loop = g_main_loop_new (NULL, TRUE);
-  fail_unless (loop != NULL);
-
-  bus = gst_element_get_bus (pipeline);
-  fail_unless (bus != NULL);
-  bus_watch = gst_bus_add_watch (bus, bus_handler, loop);
-  gst_object_unref (bus);
-
-  fail_unless_equals_int (gst_element_set_state (pipeline, GST_STATE_PAUSED),
-      GST_STATE_CHANGE_SUCCESS);
-
-  fail_unless (gst_element_seek (pipeline, 1.0, GST_FORMAT_TIME,
-          GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, 0, GST_SEEK_TYPE_SET,
-          400 * GST_MSECOND));
-
-  n_buffers = 0;
-  fail_unless_equals_int (gst_element_set_state (pipeline, GST_STATE_PLAYING),
-      GST_STATE_CHANGE_SUCCESS);
-
-  g_main_loop_run (loop);
-
-  fail_unless (test_buffer != NULL);
-  fail_unless (n_buffers >= 1);
-
-  gst_element_set_state (pipeline, GST_STATE_NULL);
-
-  gst_buffer_unref (test_buffer);
-  test_buffer = NULL;
-
-  gst_object_unref (pipeline);
-  g_main_loop_unref (loop);
-  gst_caps_unref (caps1);
-  gst_caps_unref (caps2);
-  g_source_remove (bus_watch);
-}
-
-GST_END_TEST;
-
 GST_START_TEST (test_imagefreeze_eos)
 {
   GstElement *pipeline;
@@ -568,11 +507,19 @@ GST_START_TEST (test_imagefreeze_eos)
   GstFormat fmt = GST_FORMAT_TIME;
   gint64 position;
   guint bus_watch = 0;
+  GstVideoInfo i1, i2;
 
-  caps1 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 25, 1, 1, 1);
-  caps2 =
-      gst_video_format_new_caps (GST_VIDEO_FORMAT_xRGB, 640, 480, 25, 1, 1, 1);
+  gst_video_info_init (&i1);
+  gst_video_info_set_format (&i1, GST_VIDEO_FORMAT_xRGB, 640, 480);
+  i1.fps_n = 25;
+  i1.fps_d = 1;
+  caps1 = gst_video_info_to_caps (&i1);
+
+  gst_video_info_init (&i2);
+  gst_video_info_set_format (&i2, GST_VIDEO_FORMAT_xRGB, 640, 480);
+  i2.fps_n = 25;
+  i2.fps_d = 1;
+  caps2 = gst_video_info_to_caps (&i2);
 
   pipeline = setup_imagefreeze (caps1, caps2, NULL, NULL);
 
@@ -600,7 +547,7 @@ GST_START_TEST (test_imagefreeze_eos)
 
   g_main_loop_run (loop);
 
-  fail_unless (gst_element_query_position (src, &fmt, &position));
+  fail_unless (gst_element_query_position (src, fmt, &position));
   fail_unless_equals_uint64 (position, 40 * GST_MSECOND);
 
   gst_element_set_state (pipeline, GST_STATE_NULL);
@@ -631,7 +578,6 @@ imagefreeze_suite (void)
   tcase_add_test (tc_chain, test_imagefreeze_25_1_400ms_0ms);
   tcase_add_test (tc_chain, test_imagefreeze_25_1_220ms_380ms);
 
-  tcase_add_test (tc_chain, test_imagefreeze_bufferalloc);
   tcase_add_test (tc_chain, test_imagefreeze_eos);
 
   return s;
